from typing import Optional, Any, Dict
from abaqus.Mdb.MdbCommands import upgradeMdb

def execfile(
    filename: str,
    globals: Optional[Dict[str, Any]] = None,
    locals: Optional[Dict[str, Any]] = None,
) -> None:
    """Read and execute a Python script from a file.
    The globals and locals are dictionaries, defaulting to the current
    globals and locals.  If only globals is given, locals defaults to it.

    Parameters
    ----------
    filename : os.PathLike
    globals : Optional[dict], optional, by default None
    locals : Optional[dict], optional, by default None
    """
    ...


def execPyFile(
    filePath: str,
    globs: Optional[Dict[str, Any]] = None,
    locs: Optional[Dict[str, Any]] = None,
    atxPort: int = -1,
) -> None:
    """Similar to execfile, but will run a compiled .pyc file
    The default globals and locals must be those of the caller
    Reference:
    http://www.python.org/search/hypermail/python-recent/0186.html
    If atxPort != -1, being used to run Kernel from external tester

    Parameters
    ----------
    filePath : os.PathLike
    globs : Optional[dict], optional, by default None
    locs : Optional[dict], optional, by default None
    atxPort : int, optional, by default -1
    """
    ...


def raw_input(prompt: str = "") -> str:
    """Read a string from standard input.  The trailing newline is stripped.
    If the user hits EOF (Unix: Ctl-D, Windows: Ctl-Z+Return), raise EOFError.
    On Unix, GNU readline is used if enabled.  The prompt string, if given,
    is printed without a trailing newline before reading.

    Parameters
    ----------
    prompt : str, optional
        The prompt written to standard output, by default ''

    Returns
    -------
    str
        The input value converted to a string
    """
    return input(prompt)

def cliCommand(text: str) -> None:
    """Excutes Abaqus/CAE CLI command
    
    called from cmdK_CommandDeliveryRole::sendCliCommand
    
    Parameters
    ----------
    text : str
        Command
    """


from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .GeometricRestriction import GeometricRestriction
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import AUTO, DEMOLD_REGION, SymbolicConstant


@abaqus_class_doc
class TopologyDemoldControl(GeometricRestriction):
    """The TopologyDemoldControl object defines a topology demold control geometric
    restriction.
    The TopologyDemoldControl object is derived from the GeometricRestriction object.

    .. note:: 
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].geometricRestrictions[name]
    """

    #: A String specifying the geometric restriction repository key.
    name: str

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the geometric restriction is applied.
    #: When used with a TopologyTask, there is no default value. When used with a ShapeTask,
    #: the default value is MODEL.
    region: Region

    #: None or a DatumCsys object specifying the local coordinate system of the
    #: **pullDirection**. If **csys** = None, the global coordinate system is used. When this member
    #: is queried, it returns an Int indicating the identifier of the DatumCsys. The default
    #: value is None.
    csys: Optional[int] = None

    #: A Float specifying the draft angle. The default value is 0.0.
    draftAngle: float = 0

    #: The SymbolicConstant DEMOLD_REGION or a Region object specifying the collision check
    #: region. If the value is DEMOLD_REGION, then the value of **region** is used as both the
    #: demold region and the collision check region. The default value is DEMOLD_REGION.
    collisionCheckRegion: SymbolicConstant = DEMOLD_REGION

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the point on a plane perpendicular to the pull direction,
    #: used to specify the central plane when **technique** is POINT.
    pointRegion: Optional[Region] = None

    #: A :py:class:`~abaqus.BasicGeometry.VertexArray.VertexArray` object of length 2 specifying the demold pull direction. Instead of
    #: through a ConstrainedSketchVertex, each point may be specified through a tuple of coordinates.
    pullDirection: tuple = ()

    #: A SymbolicConstant specifying the demold technique. Possible values are AUTO,
    #: AUTO_TIGHT, POINT, SURFACE, and STAMP. The default value is AUTO.
    technique: SymbolicConstant = AUTO

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        region: Region,
        csys: Optional[int] = None,
        draftAngle: float = 0,
        collisionCheckRegion: SymbolicConstant = DEMOLD_REGION,
        pointRegion: Optional[Region] = None,
        pullDirection: tuple = (),
        technique: SymbolicConstant = AUTO,
    ):
        """This method creates a TopologyDemoldControl object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].TopologyDemoldControl

        Parameters
        ----------
        name
            A String specifying the geometric restriction repository key.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the geometric restriction is applied.
            When used with a TopologyTask, there is no default value. When used with a ShapeTask,
            the default value is MODEL.
        csys
            None or a DatumCsys object specifying the local coordinate system of the
            **pullDirection**. If **csys** = None, the global coordinate system is used. When this member
            is queried, it returns an Int indicating the identifier of the DatumCsys. The default
            value is None.
        draftAngle
            A Float specifying the draft angle. The default value is 0.0.
        collisionCheckRegion
            The SymbolicConstant DEMOLD_REGION or a Region object specifying the collision check
            region. If the value is DEMOLD_REGION, then the value of **region** is used as both the
            demold region and the collision check region. The default value is DEMOLD_REGION.
        pointRegion
            A :py:class:`~abaqus.Region.Region.Region` object specifying the point on a plane perpendicular to the pull direction,
            used to specify the central plane when **technique** is POINT.
        pullDirection
            A :py:class:`~abaqus.BasicGeometry.VertexArray.VertexArray` object of length 2 specifying the demold pull direction. Instead of
            through a ConstrainedSketchVertex, each point may be specified through a tuple of coordinates.
        technique
            A SymbolicConstant specifying the demold technique. Possible values are AUTO,
            AUTO_TIGHT, POINT, SURFACE, and STAMP. The default value is AUTO.

        Returns
        -------
        TopologyDemoldControl
            A :py:class:`~abaqus.Optimization.TopologyDemoldControl.TopologyDemoldControl` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        csys: Optional[int] = None,
        draftAngle: float = 0,
        collisionCheckRegion: SymbolicConstant = DEMOLD_REGION,
        pointRegion: Optional[Region] = None,
        pullDirection: tuple = (),
        technique: SymbolicConstant = AUTO,
    ):
        """This method modifies the TopologyDemoldControl object.

        Parameters
        ----------
        csys
            None or a DatumCsys object specifying the local coordinate system of the
            **pullDirection**. If **csys** = None, the global coordinate system is used. When this member
            is queried, it returns an Int indicating the identifier of the DatumCsys. The default
            value is None.
        draftAngle
            A Float specifying the draft angle. The default value is 0.0.
        collisionCheckRegion
            The SymbolicConstant DEMOLD_REGION or a Region object specifying the collision check
            region. If the value is DEMOLD_REGION, then the value of **region** is used as both the
            demold region and the collision check region. The default value is DEMOLD_REGION.
        pointRegion
            A :py:class:`~abaqus.Region.Region.Region` object specifying the point on a plane perpendicular to the pull direction,
            used to specify the central plane when **technique** is POINT.
        pullDirection
            A :py:class:`~abaqus.BasicGeometry.VertexArray.VertexArray` object of length 2 specifying the demold pull direction. Instead of
            through a ConstrainedSketchVertex, each point may be specified through a tuple of coordinates.
        technique
            A SymbolicConstant specifying the demold technique. Possible values are AUTO,
            AUTO_TIGHT, POINT, SURFACE, and STAMP. The default value is AUTO.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .BoundaryCondition import BoundaryCondition
from ..Amplitude.Correlation import Correlation
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import Boolean, OFF, SymbolicConstant, UNSET


@abaqus_class_doc
class AccelerationBaseMotionBC(BoundaryCondition):
    """The AccelerationBaseMotionBC object stores the data for an acceleration base motion
    boundary condition.
    The AccelerationBaseMotionBC object is derived from the BoundaryCondition object.

    .. note:: 
        This object can be accessed by::

            import load
            mdb.models[name].boundaryConditions[name]
    """

    #: A String specifying the boundary condition repository key.
    name: str = ""

    #: A Float specifying the scale factor for the amplitude curve. The default value is 1.0.
    amplitudeScaleFactor: float = 1

    #: A Boolean specifying whether to define the imaginary (out-of-plane) portion of the base
    #: motion record given by amplitude definition. The default value is OFF.
    useComplex: Boolean = OFF

    #: A :py:class:`~abaqus.BasicGeometry.ModelDot.ModelDot` object specifying a tuple containing one center of rotation. The default
    #: value is the global origin. This argument applies only when **dof** = UR1, UR2, or UR3.
    centerOfRotation: tuple = ()

    #: A :py:class:`~abaqus.Amplitude.Correlation.Correlation` object.
    correlation: Correlation = Correlation()

    #: A String specifying the name of the SecondaryBaseBC object associated with this boundary
    #: condition. The default value is an empty string.
    secondaryBase: str = ""

    #: A SymbolicConstant specifying the category of the boundary condition. Possible values
    #: are MECHANICAL and THERMAL.
    category: Optional[SymbolicConstant] = None

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the boundary condition is applied.
    region: Region = Region()

    #: None or a DatumCsys object specifying the local coordinate system of the boundary
    #: condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
    #: in the global coordinate system. The default value is None.
    localCsys: Optional[str] = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        dof: SymbolicConstant,
        amplitudeScaleFactor: float = 1,
        centerOfRotation: tuple = (),
        correlation: Optional[Correlation] = None,
        secondaryBase: str = "",
        useComplex: Boolean = OFF,
        amplitude: str = UNSET,
    ):
        """This method creates a AccelerationBaseMotionBC object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].AccelerationBaseMotionBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        dof
            A SymbolicConstant specifying the constrained degree-of-freedom. Possible values for the
            SymbolicConstant are U1, U2, U3, UR1, UR2, UR3. The default value is U1.
        amplitudeScaleFactor
            A Float specifying the scale factor for the amplitude curve. The default value is 1.0.
        centerOfRotation
            A :py:class:`~abaqus.BasicGeometry.ModelDot.ModelDot` object specifying a tuple containing one center of rotation. The default
            value is the global origin. This argument applies only when **dof** = UR1, UR2, or UR3.
        correlation
            A :py:class:`~abaqus.Amplitude.Correlation.Correlation` object.
        secondaryBase
            A String specifying the name of the SecondaryBaseBC object associated with this boundary
            condition. The default value is an empty string.
        useComplex
            A Boolean specifying whether to define the imaginary (out-of-plane) portion of the base
            motion record given by amplitude definition. The default value is OFF.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.

        Returns
        -------
        AccelerationBaseMotionBC
            An :py:class:`~abaqus.BoundaryCondition.AccelerationBaseMotionBC.AccelerationBaseMotionBC` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        amplitudeScaleFactor: float = 1,
        centerOfRotation: tuple = (),
        correlation: Optional[Correlation] = None,
        secondaryBase: str = "",
        useComplex: Boolean = OFF,
        amplitude: str = UNSET,
    ):
        """This method modifies the data for an existing AccelerationBaseMotionBC object in the
        step where it is created.

        Parameters
        ----------
        amplitudeScaleFactor
            A Float specifying the scale factor for the amplitude curve. The default value is 1.0.
        centerOfRotation
            A :py:class:`~abaqus.BasicGeometry.ModelDot.ModelDot` object specifying a tuple containing one center of rotation. The default
            value is the global origin. This argument applies only when **dof** = UR1, UR2, or UR3.
        correlation
            A :py:class:`~abaqus.Amplitude.Correlation.Correlation` object.
        secondaryBase
            A String specifying the name of the SecondaryBaseBC object associated with this boundary
            condition. The default value is an empty string.
        useComplex
            A Boolean specifying whether to define the imaginary (out-of-plane) portion of the base
            motion record given by amplitude definition. The default value is OFF.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(self, stepName: str, amplitude: str = ""):
        """This method modifies the propagating data for an existing AccelerationBaseMotionBC
        object in the specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the boundary condition is modified.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous analysis step. FREED should be used if the
            boundary condition is changed to have no amplitude reference. You should provide the
            **amplitude** argument only if it is valid for the specified step.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc
from ..UtilityAndView.abaqusConstants import SymbolicConstant


@abaqus_class_doc
class OdbDataDatumCsys:
    """The OdbDataDatumCsys object stores coordinate system data.

    .. note:: 
        This object can be accessed by::

            import visualization
            session.odbData[name].datumCsyses[i]
    """

    #: A String specifying the coordinate system name. This String is read-only.
    name: str = ""

    #: A SymbolicConstant specifying the coordinate system type. This String is read-only.
    #: Possible values are CARTESIAN, CYLINDRICAL, and SPHERICAL.
    type: Optional[SymbolicConstant] = None

    #: A tuple of three Floats specifying a sequence of three floats specifying the x-Axis
    #: vector. The default value is (1, 0, 0).
    xAxis: tuple = ()

    #: A tuple of three Floats specifying a sequence of three floats specifying the y-Axis
    #: vector. The default value is (0, 1, 0).
    yAxis: tuple = ()

    #: A tuple of three Floats specifying a sequence of three floats specifying the z-Axis
    #: vector. The default value is (0, 0, 1).
    zAxis: tuple = ()

    #: A tuple of three Floats specifying a sequence of three floats specifying the origin. The
    #: default value is (0, 0, 0).
    origin: tuple = ()



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import Boolean, ON


@abaqus_class_doc
class InteractionDisplayOptions:
    """The InteractionDisplayOptions object stores settings that specify how assemblies are to
    be displayed in a particular viewport when
    session.viewports[name].assemblyDisplay.interactions=ON
    The InteractionDisplayOptions object has no constructor. When you create a new viewport,
    the settings are copied from the current viewport.

    .. note::
        This object can be accessed by::

            session.viewports[name].assemblyDisplay.interactionOptions
            session.viewports[name].layers[name].assemblyDisplay.interactionOptions
    """

    @abaqus_method_doc
    def setValues(
        self,
        surfaceContact: Boolean = ON,
        selfContact: Boolean = ON,
        elasticFoundation: Boolean = ON,
        actuatorSensor: Boolean = ON,
        radiationAmbient: Boolean = ON,
        filmCondition: Boolean = ON,
        concentratedRadiationToAmbient: Boolean = ON,
        concentratedFilmCondition: Boolean = ON,
    ):
        """This method modifies the InteractionDisplayOptions object.

        Parameters
        ----------
        surfaceContact
            A Boolean specifying whether surface contact symbols are shown. The default value is ON.
        selfContact
            A Boolean specifying whether self contact symbols are shown. The default value is ON.
        elasticFoundation
            A Boolean specifying whether elastic foundation symbols are shown. The default value is
            ON.
        actuatorSensor
            A Boolean specifying whether actuator/sensor symbols are shown. The default value is ON.
        radiationAmbient
            A Boolean specifying whether surface radiation-to-ambient symbols are shown. The default
            value is ON.
        filmCondition
            A Boolean specifying whether surface film condition symbols are shown. The default value
            is ON.
        concentratedRadiationToAmbient
            A Boolean specifying whether concentrated radiation-to-ambient symbols are shown. The
            default value is ON.
        concentratedFilmCondition
            A Boolean specifying whether concentrated film condition symbols are shown. The default
            value is ON.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from typing import Optional, Tuple, overload

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ConstrainedSketchConstraint.ConstrainedSketchConstraintModel import (
    ConstrainedSketchConstraintModel,
)
from .ConstrainedSketchDimension.ConstrainedSketchDimensionModel import (
    ConstrainedSketchDimensionModel,
)
from .ConstrainedSketchGeometry.ConstrainedSketchGeometry import (
    ConstrainedSketchGeometry,
)
from .ConstrainedSketchGeometry.ConstrainedSketchGeometryModel import (
    ConstrainedSketchGeometryModel,
)
from .ConstrainedSketchParameter.ConstrainedSketchParameterModel import (
    ConstrainedSketchParameterModel,
)
from .ConstrainedSketchVertex.ConstrainedSketchVertex import ConstrainedSketchVertex
from .ConstrainedSketchVertex.ConstrainedSketchVertexModel import (
    ConstrainedSketchVertexModel,
)
from ..Part.AcisFile import AcisFile
from ..UtilityAndView.SymbolicConstant import abaqusConstants as C
from ..UtilityAndView.abaqusConstants import (Boolean, OFF)


@abaqus_class_doc
class ConstrainedSketch(
    ConstrainedSketchConstraintModel,
    ConstrainedSketchDimensionModel,
    ConstrainedSketchGeometryModel,
    ConstrainedSketchParameterModel,
    ConstrainedSketchVertexModel,
):
    @overload
    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        sheetSize: float,
        gridSpacing: Optional[float] = None,
        transform: tuple = (),
    ) -> None:
        """This method creates a ConstrainedSketch object. If the sketch cannot be created, the
        method returns None.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].ConstrainedSketch

        Parameters
        ----------
        name
            A String specifying the repository key.
        sheetSize
            A Float specifying the sheet size.
        gridSpacing
            A Float specifying the spacing between gridlines. Possible values are Floats >> 0. The
            default value is approximately 2 percent of **sheetSize**.
        transform
            A sequence of sequences of Floats specifying the three-dimensional orientation of the
            sketch. The sequence is a 3 × 4 transformation matrix specifying the axis of rotation
            and the translation vector. Possible values are any Floats.The default value for the
            axis of rotation is the identity matrix`(1.0, 0.0, 0.0),  (0.0, 1.0, 0.0),  (0.0, 0.0,
            1.0)`The default value for the translation vector is`(0.0, 0.0, 0.0)`The default values
            position the sketch on the **X - Y** plane centered at the origin.

        Returns
        -------
        sketch: ConstrainedSketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object
        """
        ...

    @overload
    @abaqus_method_doc
    def __init__(self, name: str, objectToCopy: ConstrainedSketch) -> None:
        """This method copies one ConstrainedSketch object to a new ConstrainedSketch object. Note:
        If the name of the sketch to be copied to is __edit__, Abaqus creates an exact copy that
        contains both reference geometry and a non-identity transform matrix. Otherwise, the
        Sketch copy constructor strips the reference geometry from the copied sketch and sets
        the transform matrix to identity, creating a standalone copy.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].ConstrainedSketch

        Parameters
        ----------
        name
            A String specifying the repository key.
        objectToCopy
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object to be copied.

        Returns
        -------
        sketch: ConstrainedSketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object
        """
        ...

    @abaqus_method_doc
    def __init__(self, *args, **kwargs) -> None:
        ...

    def ConstrainedSketchFromGeometryFile(
        self, name: str, geometryFile: AcisFile
    ) -> ConstrainedSketch:
        """This method creates a ConstrainedSketch object and places it in the sketches repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].ConstrainedSketchFromGeometryFile

        Parameters
        ----------
        name
            A String specifying the repository key.
        geometryFile
            An :py:class:`~abaqus.Part.AcisFile.AcisFile` object specifying a file containing geometry. The geometry in the file is
            converted to two-dimensional sketch geometry in the **X - Y** plane.

        Returns
        -------
        sketch: ConstrainedSketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object
        """
        ...

    @abaqus_method_doc
    def print(self) -> None:
        """This method prints the following statistics about a sketch:
        - The sketch Id (a positive integer).
        - The number of geometry curves (the number of ConstrainedSketchGeometry objects).
        - The number of dimensions (the number of ConstrainedSketchDimension objects).
        - The number of vertices (the number of ConstrainedSketchVertex objects).
        """
        ...

    @abaqus_method_doc
    def assignCenterline(self, line: ConstrainedSketchGeometry) -> None:
        """This method indicates the construction line that will be used as a centerline for
        revolved features.

        Parameters
        ----------
        line
            A :py:class:`~abaqus.Sketcher.ConstrainedSketchGeometry.ConstrainedSketchGeometry.ConstrainedSketchGeometry` object specifying a construction line that indicates the
            centerline of revolved features.
        """
        ...

    @abaqus_method_doc
    def assignCenterOfTwist(self, point: ConstrainedSketchVertex) -> None:
        """This method indicates the isolated point that will be used as the center of twist when
        an extruded feature is created with twist.

        Parameters
        ----------
        point
            A :py:class:`~abaqus.Sketcher.ConstrainedSketchVertex.ConstrainedSketchVertex.ConstrainedSketchVertex` object specifying an isolated point that indicates the center
            of twist for extruded features that use a twist angle.
        """
        ...

    @abaqus_method_doc
    def autoDimension(self, objectList: tuple) -> None:
        """This method applies dimensions to the selected ConstrainedSketchGeometry objects in an
        effort to make the ConstrainedSketch well defined.

        Parameters
        ----------
        objectList
            A sequence specifying the ConstrainedSketchGeometry objects to dimension.
        """
        ...

    @abaqus_method_doc
    def autoTrimCurve(
        self, curve1: str, point1: Tuple[float, float], parameter1: float
    ) -> None:
        """This method automatically trims a selected ConstrainedSketchGeometry object at the
        specified location. If the object does not intersect other ConstrainedSketchGeometry
        objects, the entire selected object will be deleted.

        Parameters
        ----------
        curve1
            The ConstrainedSketchGeometry object to be trimmed.
        point1
            A pair of Floats specifying the location on ConstrainedSketchGeometry where the trimming
            should be applied. **point1** and **parameter1** are mutually exclusive.
        parameter1
            A Float specifying the parameter location on the ConstrainedSketchGeometry where the
            trimming should be applied. **point1** and **parameter1** are mutually exclusive.
        """
        ...

    @abaqus_method_doc
    def breakCurve(
        self,
        curve1: ConstrainedSketchGeometry,
        point1: Tuple[float, float],
        curve2: ConstrainedSketchGeometry,
        point2: Tuple[float, float],
    ) -> None:
        """This method breaks a specified ConstrainedSketchGeometry object (*curve1*) using another
        specified ConstrainedSketchGeometry object (*curve2*). If the selected
        ConstrainedSketchGeometry objects intersect, then only **curve1** will be broken; **curve2**
        is not affected by the operation. The location for the break is determined by the
        specified point values.

        Parameters
        ----------
        curve1
            A :py:class:`~abaqus.Sketcher.ConstrainedSketchGeometry.ConstrainedSketchGeometry.ConstrainedSketchGeometry` object specifying the object to be broken.
        point1
            A pair of Floats specifying the location on **curve1** near where the break should be
            applied.
        curve2
            A :py:class:`~abaqus.Sketcher.ConstrainedSketchGeometry.ConstrainedSketchGeometry.ConstrainedSketchGeometry` object specifying where **curve1** should be broken.
        point2
            A pair of Floats specifying the location on **curve2** near where **curve1** should be
            broken.
        """
        ...

    @abaqus_method_doc
    def copyMirror(
        self,
        mirrorLine: ConstrainedSketchGeometry,
        objectList: Tuple[ConstrainedSketchGeometry, ...],
    ) -> None:
        """This method creates copies of the given ConstrainedSketchGeometry objects, mirrors them
        about a selected line, and inserts them into the appropriate repositories of the
        ConstrainedSketch object.

        Parameters
        ----------
        mirrorLine
            A :py:class:`~abaqus.Sketcher.ConstrainedSketchGeometry.ConstrainedSketchGeometry.ConstrainedSketchGeometry` object specifying the line about which Abaqus will mirror
            the sketch.
        objectList
            A sequence of ConstrainedSketchGeometry objects specifying the sketch to be copied and
            mirrored.
        """
        ...

    @abaqus_method_doc
    def copyMove(
        self, vector: tuple, objectList: Tuple[ConstrainedSketchGeometry, ...]
    ) -> None:
        """This method creates copies of the given ConstrainedSketchGeometry objects, moves them
        from their original position, and inserts them into the appropriate repositories of the
        ConstrainedSketch object.

        Parameters
        ----------
        vector
            A sequence of two Floats specifying the translation vector.
        objectList
            A sequence of ConstrainedSketchGeometry objects to be copied and moved.
        """
        ...

    @abaqus_method_doc
    def copyRotate(
        self,
        centerPoint: Tuple[float, float],
        angle: float,
        objectList: Tuple[ConstrainedSketchGeometry, ...],
    ) -> None:
        """This method creates copies of the given ConstrainedSketchGeometry objects, rotates them,
        and inserts them into the appropriate repositories of the ConstrainedSketch object.

        Parameters
        ----------
        centerPoint
            A pair of Floats specifying the center of rotation.
        angle
            A Float specifying the angle of rotation in degrees.
        objectList
            A sequence of ConstrainedSketchGeometry objects to be copied and moved.
        """
        ...

    @abaqus_method_doc
    def copyScale(
        self,
        scaleValue: float,
        scaleCenter: Tuple[float, float],
        objectList: Tuple[ConstrainedSketchGeometry, ...],
    ) -> None:
        """This method creates copies of the given ConstrainedSketchGeometry objects, scales them
        by the specified value about a selected point, and inserts them into the appropriate
        repositories of the ConstrainedSketch object.

        Parameters
        ----------
        scaleValue
            A Float specifying the value for scaling.
        scaleCenter
            A pair of Floats specifying the center of scaling.
        objectList
            A sequence of ConstrainedSketchGeometry objects to be copied and scaled.
        """
        ...

    @abaqus_method_doc
    def delete(self, objectList: tuple) -> None:
        """This method deletes the given ConstrainedSketchGeometry, ConstrainedSketchDimension, or
        ConstrainedSketchConstraint objects.

        Parameters
        ----------
        objectList
            A sequence of ConstrainedSketchGeometry, ConstrainedSketchDimension, or
            ConstrainedSketchConstraint objects to be deleted.
        """
        ...

    @abaqus_method_doc
    def deleteParameter(self, name: str) -> None:
        """The command deletes a specified parameter.

        Parameters
        ----------
        name
            A String specifying the name of the parameter to delete.
        """
        ...

    @abaqus_method_doc
    def dragEntity(self, entity: str, points: tuple) -> None:
        """This method drags a specified ConstrainedSketchGeometry or ConstrainedSketchVertex
        object to a specific location.

        Parameters
        ----------
        entity
            A ConstrainedSketchGeometry or ConstrainedSketchVertex object specifying the object to
            drag.
        points
            A sequence of sequences of three Floats specifying a sequence of points along which to
            drag the entity. The order of points in the sequence defines a path that determines the
            solution.
        """
        ...

    @abaqus_method_doc
    def linearPattern(
        self,
        number1: int,
        spacing1: float,
        angle1: float,
        vertexList: Tuple[ConstrainedSketchVertex, ...] = (),
        geomList: Tuple[ConstrainedSketchGeometry, ...] = (),
        number2: int = 1,
        spacing2: Optional[float] = None,
        angle2: Optional[float] = None,
    ) -> None:
        """This method copies ConstrainedSketchGeometry objects in a linear pattern along one or
        two directions. This method also copies any associated dimension or constraint objects
        that exist between the given objects.

        Parameters
        ----------
        number1
            An Integer specifying the total number of copies, including the original objects, that
            appear along the first direction in the pattern. Possible values are 1 ≤ **number1** ≤
            1000.
        spacing1
            A Float specifying the spacing between copies along the first direction in the pattern.
            Possible values are 0.0 ≤ **spacing1** .
        angle1
            A Float specifying the angle in degrees of the first direction in the pattern. Possible
            values are -360.0 ≤ **angle1** ≤ 360.0.
        vertexList
            A sequence of ConstrainedSketchVertex objects to copy.
        geomList
            A sequence of ConstrainedSketchGeometry objects to copy.
        number2
            An integer specifying the total number of copies, including the original objects, that
            appear along the second direction in the pattern. Possible values are 1 ≤ **number2** ≤
            1000. The default value is 1. The value of either **number1** or **number2** must be greater
            than one.
        spacing2
            A Float specifying the spacing between copies along the first direction in the pattern.
            Possible values are 0.0 ≤ **spacing2**. The default value is **spacing1**.
        angle2
            A Float specifying the angle in degrees of the first direction in the pattern. Possible
            values are -360.0 ≤ **angle2** ≤ 360.0. The default value is 90° beyond the value of
            **angle1**.

        Returns
        -------
        None.

        Raises
        ------
        AbaqusException
            Number must be greater than 1 for at least one direction
        """
        ...

    @abaqus_method_doc
    def mergeVertices(
        self, value: float, vertexList: Tuple[ConstrainedSketchVertex, ...]
    ) -> None:
        """This method merges the ConstrainedSketchVertex objects that lie within the specified
        distance of each other. If only one ConstrainedSketchVertex object is selected, it will
        merge all ConstrainedSketchVertex objects that lie within the specified distance of that
        vertex. If more than one vertex is selected, the search will be restricted to only the
        selected ConstrainedSketchVertex objects.

        Parameters
        ----------
        value
            A Float specifying the search radius.
        vertexList
            A sequence of ConstrainedSketchVertex objects to be merged.
        """
        ...

    @abaqus_method_doc
    def move(
        self, vector: tuple, objectList: Tuple[ConstrainedSketchGeometry, ...]
    ) -> None:
        """This method translates the given ConstrainedSketchGeometry objects by the given vector.

        Parameters
        ----------
        vector
            A sequence of two Floats specifying the translation vector.
        objectList
            A sequence of ConstrainedSketchGeometry objects specifying the objects to be translated.
        """
        ...

    @abaqus_method_doc
    def offset(
        self,
        distance: float,
        objectList: Tuple[ConstrainedSketchGeometry, ...],
        side: Literal[C.LEFT, C.RIGHT],
        filletCorners: Boolean = OFF,
    ) -> None:
        """This method creates copies of the selected ConstrainedSketchGeometry objects, offsets
        them by the specified distance in the specified direction, and inserts them into the
        ConstrainedSketch object's appropriate repositories. If connected objects are selected,
        trim or extend is carried out to complete the offset.

        Parameters
        ----------
        distance
            A Float specifying the distance to be offset.
        objectList
            A sequence of ConstrainedSketchGeometry objects to be copied and offset.
        side
            A SymbolicConstant specifying which side the offset should occur. Possible values are
            LEFT and RIGHT.
        filletCorners
            A Boolean specifying whether the corners need to be rounded instead of being extended.
        """
        ...

    @abaqus_method_doc
    def radialPattern(
        self,
        number: int,
        totalAngle: float,
        centerPoint: Tuple[float, float],
        vertexList: Tuple[ConstrainedSketchVertex, ...] = (),
        geomList: Tuple[ConstrainedSketchGeometry, ...] = (),
    ) -> None:
        """This method copies ConstrainedSketchGeometry objects in a radial pattern about a
        specified center point.

        Parameters
        ----------
        number
            An Int specifying the total number of copies, including the original objects, that
            appear in the radial pattern. Possible values are 2 ≤ **number2** ≤ 1000.
        totalAngle
            A Float specifying the total angle in degrees between the first and last instance in the
            pattern. A positive angle corresponds to a counter-clockwise direction. The values 360°
            and -360° represent a special case where the pattern makes a full circle. In this case,
            because the copy would overlay the original, the copy is not placed at the last
            position. Possible values are -360.0 ≤ **totalAngle** ≤ 360.0.
        centerPoint
            A pair of Floats specifying the center of the radial pattern.
        vertexList
            A sequence of ConstrainedSketchVertex objects to copy.
        geomList
            A sequence of ConstrainedSketchGeometry objects to copy.
        """
        ...

    @abaqus_method_doc
    def resetView(self) -> None:
        """This method resets the view to be perpendicular to the sketching plane."""
        ...

    @abaqus_method_doc
    def rectangle(
        self, point1: Tuple[float, float], point2: Tuple[float, float]
    ) -> int:
        """This method creates four lines that form a rectangle with diagonal corners defined by
        the given points and inserts them into the geometry repository of the ConstrainedSketch
        object.

        Parameters
        ----------
        point1
            A pair of Floats specifying the first corner of the rectangle.
        point2
            A pair of Floats specifying the second corner of the rectangle.

        Returns
        -------
        success: int
            An Int specifying the success or failure of the method. A value of 0 indicates failure
        """
        ...

    @abaqus_method_doc
    def removeGapsAndOverlaps(
        self, tolerance: str, geomList: Tuple[ConstrainedSketchGeometry, ...]
    ) -> None:
        """This method removes gaps and overlaps between sketch geometries specified by the user.
        This method is particularly useful when cleaning up imported sketches

        Parameters
        ----------
        tolerance
            A float value which specifies the largest size of the gap or overlap between entities
            that is to be removed. Typically this value is small and is used to close gaps and
            overlaps which may not exist in the originating program but exist in the sketch because
            of mismatched tolerances between the two programs.
        geomList
            A sequence of ConstrainedSketchGeometry objects where the gaps and overlaps are to be
            removed.
        """
        ...

    @abaqus_method_doc
    def repairShortEdges(
        self, geomList: Tuple[ConstrainedSketchGeometry, ...], tolerance: str = ""
    ) -> None:
        """This method deletes the short edges specified, optionally selecting only those short
        edges whose lengths are smaller than the specified tolerance and healing the resultant
        gap in the sketch. This method is particularly useful in conjunction with
        removeGapsAndOverlap when cleaning up imported sketches.

        Parameters
        ----------
        geomList
            A sequence of ConstrainedSketchGeometry objects where the short edges are to be removed.
        tolerance
            A float value that is used to select and delete only those edges specified in **geomList**
            whose lengths are smaller than the given value. The default value is -1.0. This value
            implies that all edges specified in **geomList** will be removed and the sketch healed to
            remove gaps left by their removal.
        """
        ...

    @abaqus_method_doc
    def retrieveSketch(self, sketch: ConstrainedSketch) -> None:
        """This method copies all ConstrainedSketchGeometry, ConstrainedSketchDimension,
        ConstrainedSketchConstraint, and ConstrainedSketchParameter objects from the specified
        ConstrainedSketch object. The new objects are added to the existing objects (if any).
        The objects in the specified ConstrainedSketch object are not modified by the retrieve
        operation.

        Parameters
        ----------
        sketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object specifying the object from which to copy.
        """
        ...

    @abaqus_method_doc
    def rotate(
        self, centerPoint: Tuple[float, float], angle: float, objectList: tuple
    ) -> None:
        """This method rotates the given ConstrainedSketchGeometry objects by the given angle and
        about the given point.

        Parameters
        ----------
        centerPoint
            A pair of Floats specifying the center of rotation.
        angle
            A Float specifying the angle of rotation in degrees.
        objectList
            A sequence of ConstrainedSketchGeometry specifying the objects to be rotated.
        """
        ...

    @abaqus_method_doc
    def scale(
        self,
        scaleValue: float,
        scaleCenter: Tuple[float, float],
        objectList: Tuple[ConstrainedSketchGeometry, ...],
    ) -> None:
        """This method scales the given ConstrainedSketchGeometry objects by the given scale factor
        and about the given point.

        Parameters
        ----------
        scaleValue
            A Float specifying the value of scale.
        scaleCenter
            A pair of Floats specifying the center of scale.
        objectList
            A sequence of ConstrainedSketchGeometry objects specifying the objects to be scaled.
        """
        ...

    @abaqus_method_doc
    def setPrimaryObject(self, option: Literal[C.STANDALONE, C.SUPERIMPOSE]) -> None:
        """This method makes the ConstrainedSketch object the primary object in the current
        viewport. The sketch remains the primary object in the current viewport until an
        unsetPrimaryobject command is issued.

        Parameters
        ----------
        option
            A SymbolicConstant specifying how the sketch is displayed. Possible values are:
            
            * `STANDALONE`: Indicates a new stand-alone sketch. The current viewport is
                cleared and is replaced by the stand-alone sketch. The view direction
                is set to -Z.

            * `SUPERIMPOSE`: Indicates that the sketch is superimposed on the current
                viewport. The view direction is changed to be perpendicular to the
                sketch plane. The change is effected smoothly as an animated sequence
                of many small viewing steps.
        """
        ...

    @abaqus_method_doc
    def trimExtendCurve(
        self,
        curve1: str,
        point1: Tuple[float, float],
        curve2: str,
        point2: Tuple[float, float],
    ) -> None:
        """This method trims or extends a specified ConstrainedSketchGeometry object (*curve1*)
        using another specified ConstrainedSketchGeometry object (*curve2*). **curve2** is not
        affected by the operation. The location for the trim or extend is determined by the
        specified point values.

        Parameters
        ----------
        curve1
            The ConstrainedSketchGeometry object specifying the object to be trimmed or extended.
        point1
            A pair of Floats specifying the location on **curve1** where trim or extend should be
            applied.
        curve2
            The ConstrainedSketchGeometry object specifying the object to which **curve1** is trimmed
            or extended. **curve2** is not trimmed or extended.
        point2
            A pair of Floats specifying the location on **curve2** near where **curve1** should be
            trimmed or extended.
        """
        ...

    @abaqus_method_doc
    def undo(self) -> None:
        """This method undoes the effects of the last ConstrainedSketch object method."""
        ...

    @abaqus_method_doc
    def unsetPrimaryObject(self) -> None:
        """This method removes the ConstrainedSketch object from the current viewport, reversing
        the effects of the setPrimaryobject command. If the **option** argument was set to
        SUPERIMPOSE, the viewport will be returned to the view orientation that was in place
        when the setPrimaryobject command was issued. If the **option** argument was set to
        STANDALONE, the viewport will be left empty.
        """
        ...

    @abaqus_method_doc
    def writeAcisFile(self, fileName: str, version: Optional[float] = None) -> None:
        """This method exports the geometry of the sketch to a named file in ACIS format.

        Parameters
        ----------
        fileName
            A String specifying the name of the file to which to write. The file name's extension is
            used to determine whether a part or assembly is written. Use the file extension .asat
            for the assembly format.

            .. versionchanged:: 2018
                Add description for thr file name's extension.
        version
            A Float specifying the ACIS version. For example, the Float 12.0 corresponds to ACIS
            Version 12.0. The default value is the current version of ACIS.
        """
        ...

    @abaqus_method_doc
    def writeIgesFile(
        self,
        filename: str,
        flavor: Literal[C.STANDARD, C.AUTOCAD, C.SOLIDWORKS, C.JAMA, C.MSBO] = ...,
    ) -> None:
        """This method exports the geometry of the sketch to a named file in IGES format.

        Parameters
        ----------
        filename
            A String specifying the file name.
        flavor
            A SymbolicConstant specifying a particular flavor of IGES to export. Possible values
            are STANDARD, AUTOCAD, SOLIDWORKS, JAMA, and MSBO.
        """
        ...



from typing import Optional

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import Boolean, OFF


@abaqus_class_doc
class Vertex:
    """Vertices are point regions of geometry.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].allInternalSets[name].vertices[i]
            mdb.models[name].parts[name].allSets[name].vertices[i]
            mdb.models[name].parts[name].sets[name].vertices[i]
            mdb.models[name].parts[name].vertices[i]
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].sets[name].vertices[i]
            mdb.models[name].rootAssembly.allInstances[name].vertices[i]
            mdb.models[name].rootAssembly.allInternalSets[name].vertices[i]
            mdb.models[name].rootAssembly.allSets[name].vertices[i]
            mdb.models[name].rootAssembly.instances[name].sets[name].vertices[i]
            mdb.models[name].rootAssembly.instances[name].vertices[i]
            mdb.models[name].rootAssembly.modelInstances[i].sets[name].vertices[i]
            mdb.models[name].rootAssembly.modelInstances[i].vertices[i]
            mdb.models[name].rootAssembly.sets[name].vertices[i]
            mdb.models[name].rootAssembly.vertices[i]
    """

    #: An Int specifying the index of the ConstrainedSketchVertex in the VertexArray.
    index: Optional[int] = None

    #: A Boolean specifying whether the vertex belongs to the reference representation of the
    #: Part or Instance.
    isReferenceRep: Boolean = OFF

    #: A tuple of Floats specifying the **X** -, **Y** -, and **Z** -coordinates of the vertex.
    pointOn: Optional[float] = None

    #: A tuple of Floats specifying the name of the feature that created this vertex.
    featureName: Optional[float] = None

    #: A tuple of Floats specifying the name of the part instance for this vertex (if
    #: applicable).
    instanceName: Optional[float] = None

    @abaqus_method_doc
    def getEdges(self):
        """This method returns a sequence consisting of the edge ids of the edges which share this
        vertex.

        Returns
        -------
        Tuple[int, ...]
            A tuple of integers.

        """
        ...

    @abaqus_method_doc
    def getNodes(self):
        """This method returns an array of node objects that are associated with the vertex.

        Returns
        -------
        MeshNodeArray
            A :py:class:`~abaqus.Mesh.MeshNodeArray.MeshNodeArray` object which is a sequence of MeshNode objects.

        """
        ...

    @abaqus_method_doc
    def getElements(self):
        """This method returns an array of element objects that are associated with the vertex.

        Returns
        -------
        MeshElementArray
            A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object which is a sequence of MeshElement objects.

        """
        ...
